﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.ServiceModel.Web;
using System.Text;
using System.ComponentModel;

namespace WcfRestService1
{
    // Start the service and browse to http://<machine_name>:<port>/Service1/help to view the service's generated help page
    // NOTE: By default, a new instance of the service is created for each call; change the InstanceContextMode to Single if you want
    // a single instance of the service to process all calls.	
    [ServiceContract]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall)]
    // NOTE: If the service is renamed, remember to update the global.asax.cs file
    public class Service1
    {
        [WebGet(UriTemplate = "")]
        public string Default() {
            return "Visit /help for more information";
        }

        [WebGet(UriTemplate = "Course/{courseID}")]
        [Description("Specific course by course id")]
        public Course courseByCouseID(string courseID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(courseID);

                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Courses
                            where s.CourseID == lookupID
                            select s;
                return query.ToList().First();

            }

        }

        [WebGet(UriTemplate = "Course/CourseCode/{coursecode}")]
        [Description("Specific course by course code")]
        public Course courseByCouseCode(string courseCode)
        {
            using (var context = new wsa500_103a06Entities())
            {
                string lookupID = courseCode.ToString().ToLower();
                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Courses
                            where s.CourseCode.ToLower().Equals(lookupID)
                            select s;
                return query.ToList().First();

            }

        }

        [WebGet(UriTemplate = "Program/{programID}")]
        [Description("Specific Program")]
        public Program programByProgramID(string programID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(programID);

                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Programs
                            where s.ProgramID == lookupID
                            select s;
                return query.ToList().First();

            }

        }


        [WebGet(UriTemplate = "CourseInstance/{courseInstanceID}")]
        [Description("Specific CourseInstance")]
        public CourseInstance courseInstanceBycourseInstanceID(string courseInstanceID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(courseInstanceID);

                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.CourseInstances
                            where s.CourseInstanceID == lookupID
                            select s;
                return query.ToList().First();

            }

        }

       
        [WebGet(UriTemplate = "Room/{roomID}")]
        [Description("Specific Room")]
        public Room roomByRoomID(string roomID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                string lookupID = roomID.ToString().ToLower();
                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Rooms
                            where s.RoomID.ToString().ToLower().Equals(lookupID)
                            select s;
                return query.ToList().First();

            }

        }

        [WebGet(UriTemplate = "Rooms/{buildingname}/{roomtype}")]
        [Description("Rooms of Specific roomtype in a specific building")]
        public List<Room> roomsByBuildingNameAndRoomType(string buildingname, string roomtype)
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Rooms
                            where s.Building.ToLower().Equals(buildingname.ToLower()) && s.RoomType.ToLower().Equals(roomtype.ToLower())
                            select s;
                return query.ToList();

            }

        }

        [WebGet(UriTemplate = "Rooms/{buildingname}")]
        [Description("Rooms of Specific campus")]
        public List<Room> roomsByBuildingName(string buildingname)
        {
            using (var context = new wsa500_103a06Entities())
            {
                string lookupID = buildingname.ToString().ToLower();
                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Rooms
                            where s.Building.ToString().ToLower().Equals(lookupID)
                            select s;
                return query.ToList();

            }

        }

        [WebGet(UriTemplate = "Room/{campusname}/{roomtype}")]
        [Description("Rooms of Specific roomtype in a specific campus")]
        public List<Room> roomsByCampusNameAndRoomtype(string campusName, string roomtype)
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Rooms
                            where s.Campus.ToLower().Equals(campusName.ToLower()) && s.RoomType.ToLower().Equals(roomtype.ToLower())
                            select s;
                return query.ToList();

            }

        }

        [WebGet(UriTemplate = "CourseInstanceSession/{courseInstanceSessionID}")]
        [Description("Specific CourseInstanceSession")]
        public CourseInstanceSession courseInstanceSessionBycourseInstanceSessionID(string courseInstanceSessionID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(courseInstanceSessionID);

                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.CourseInstanceSessions
                            where s.CourseInstanceSessionID == lookupID
                            select s;
                return query.ToList().First();

            }

        }


        [WebGet(UriTemplate = "JobDuty/{jobDutyID}")]
        [Description("Specific JobDuty")]
        public JobDuty jobDutyByID(string jobDutyID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(jobDutyID);

                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.JobDuties
                            where s.JobDutyID == lookupID
                            select s;
                return query.ToList().First();

            }

        }


        [WebGet(UriTemplate = "Semester/{semesterID}")]
        [Description("Specific Semester")]
        public Semester semesterByID(string semesterID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(semesterID);

                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Semesters
                            where s.SemesterID == lookupID
                            select s;
                return query.ToList().First();

            }

        }


        [WebGet(UriTemplate = "Employee/{employeeID}")]
        [Description("Specific Employee")]
         public Employee employeeByID(string employeeID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(employeeID);

                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Employees
                            where s.EmployeeID == lookupID
                            select s;
                return query.ToList().First();

            }

        }


        [WebGet(UriTemplate = "EmployeeNamed/{firstname}/{lastname}")]
        [Description("Employee with firstname and lastname")]
         public Employee employeeByFirstnameLastname(string firstname, string lastname)
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.Employees
                            where s.FirstName.ToLower().Equals(firstname.ToLower()) && s.LastName.ToLower().Equals(lastname.ToLower())
                            select s;
                return query.ToList().First();

            }

        }

        [WebGet(UriTemplate = "JobTitle/{jobTitleID}")]
        [Description("Specific JobTitle")]
         public JobTitle jobTitleByID(string jobtitleID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(jobtitleID);
                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.JobTitles
                            where s.JobTitleID == lookupID
                            select s;
                return query.ToList().First();

            }

        }


        [WebGet(UriTemplate = "CourseEvent/{courseEventID}")]
        [Description("Specific CourseEvent")]
         public CourseEvent courseEventByID(string courseEventID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(courseEventID);
                context.ContextOptions.ProxyCreationEnabled = false;

                var query = from s in context.CourseEvents
                            where s.CourseEventID == lookupID
                            select s;
                return query.ToList().First();

            }

        }


        [WebGet(UriTemplate = "Courses")]
        [Description("All courses")]
        public List<Course> courses(){
            using(var context = new wsa500_103a06Entities()){
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Courses
                            select s;
                return query.ToList();
            }
        
        }
        
        [WebGet(UriTemplate = "Programs")]
        [Description("All Programs")]
        public List<Program> programs()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Programs
                            select s;
                return query.ToList();
            }

        }

        [WebGet(UriTemplate = "CourseInstances")]
        [Description("All CourseInstances")]
        public List<CourseInstance> courseInstances()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.CourseInstances
                            select s;
                return query.ToList();
            }

        }

        [WebGet(UriTemplate = "Rooms")]
        [Description("All Rooms")]
        public List<Room> rooms()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Rooms
                            select s;
                return query.ToList();
            }

        }

        [WebGet(UriTemplate = "CourseInstanceSessions")]
        [Description("All CourseInstanceSessions")]
        public List<CourseInstanceSession> courseInstanceSessions()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.CourseInstanceSessions
                            select s;
                return query.ToList();
            }

        }

        [WebGet(UriTemplate = "JobDuties")]
        [Description("All JobDuties")]
        public List<JobDuty> jobDuties()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.JobDuties
                            select s;
                return query.ToList();
            }

        }

        [WebGet(UriTemplate = "Semesters")]
        [Description("All Semesters")]
        public List<Semester> semesters()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Semesters
                            select s;
                return query.ToList();
            }

        }
        
        [WebGet(UriTemplate = "Employees")]
        [Description("All Employees")]
          public List<Employee> employees()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Employees
                            select s;
                return query.ToList();
            }

        }

        [WebGet(UriTemplate = "Employees/Active")]
        [Description("All active Employees")]
          public List<Employee> employeesActive()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Employees
                            where s.Active == true
                            select s;
                return query.ToList();
            }

        }


        [WebGet(UriTemplate = "Employees/InActive")]
        [Description("All inactive Employees")]
         public List<Employee> employeesInactive()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Employees
                            where s.Active == false
                            select s;
                return query.ToList();
            }

        }

        [WebGet(UriTemplate = "Professors")]
        [Description("All Employees with professor jobtitle")]
          public List<Employee> professors()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from p in context.Employees
                            where p.JobTitle.JobTitleName.ToLower().Equals("Professor".ToLower())
                            select p;
                return query.ToList();
            }

        }



        [WebGet(UriTemplate = "Employees/{jobtitle}")]
        [Description("Employee with job title")]
           public List<Employee> employeesWithJobTitle(string jobtitle)
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Employees
                            where s.JobTitle.ToString().ToLower().Equals("Professor".ToLower())
                            select s;
                return query.ToList();
            }

        }


        [WebGet(UriTemplate = "JobTitles")]
        [Description("All JobTitles")]
            public List<JobTitle> jobTitles()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.JobTitles
                            select s;
                return query.ToList();
            }

        }

        [WebGet(UriTemplate = "CourseEvents")]
        [Description("All CourseEvents")]
             public List<CourseEvent> courseEvents()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.CourseEvents
                            select s;
                return query.ToList();
            }

        }

        [WebGet(UriTemplate = "Employees/Count")]
        [Description("Total employees")]
            public int employeeCount()
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Employees
                            select s;
                return query.ToList().Count;
            }

        }

        [WebGet(UriTemplate = "Room/{campusname}/{buildingname}/Count")]
        [Description("All room count for a building name")]
        public int roomCountByCampusAndBuilding(string campusname, string buildingname)
        {
            using (var context = new wsa500_103a06Entities())
            {
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from s in context.Rooms
                            where s.Campus.ToLower().Equals(campusname.ToLower())&& s.Building.ToLower().Equals(buildingname.ToLower())
                            select s;
                return query.ToList().Count;
            }

        }




        [WebGet(UriTemplate = "Course/{courseID}/Programs")]
        [Description("Programs that include a specific course id")]
        public List<Program> ProgramsForCourse(string courseID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(courseID);
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from pc in context.ProgramCourses
                            where pc.CourseID == lookupID
                            select pc.Program as Program;
                return query.ToList();
            }
        }


       [WebGet(UriTemplate = "Program/{programID}/Courses")]
        [Description("Courses that belong to a specific program")]
        public List<Course> CoursesByProgramID(string programID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(programID);
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from pc in context.ProgramCourses
                            where pc.ProgramID == lookupID
                            select pc.Course as Course;
                return query.ToList();
            }
        }


        [WebGet(UriTemplate = "Courses/Employee/{employeeID}")]
        [Description("Courses taught by a specific Employee")]
         public List<Course> CoursesByEmployee(string employeeID)
        {
            using (var context = new wsa500_103a06Entities())
            {
                int lookupID = int.Parse(employeeID);
                context.ContextOptions.ProxyCreationEnabled = false;
                var query = from c in context.CourseInstances
                                where c.EmployeeID == lookupID
                            select c.Course as Course;
                return query.ToList();
            }
        }
        
        
       [WebGet(UriTemplate = "Rooms/Course/{courseCode}")]
        [Description("Rooms for a specific course code")]
        public List<Room> RoomsByCourseCode(string courseCode)
       {
           using (var context = new wsa500_103a06Entities())
           {
               String lookup = courseCode.ToString();
               context.ContextOptions.ProxyCreationEnabled = false;
               var query = from c in context.CourseInstanceSessions
                               where c.CourseInstance.Course.CourseCode.ToLower().Equals(lookup.ToLower())
                           select c.Room as Room;
               return query.ToList();
           }
       }
      

       [WebGet(UriTemplate = "CourseEvents/Course/{coursecode}")]
       [Description("CourseEvents for a specific course")]
       public List<CourseEvent> CourseEventsByCourseCode(string courseCode)
       {
           using (var context = new wsa500_103a06Entities())
           {
               String lookup = courseCode.ToString();
               context.ContextOptions.ProxyCreationEnabled = false;
               var query = from ce in context.CourseEvents
                           where ce.CourseInstance.Course.CourseCode.ToLower().Equals(lookup.ToLower())
                           select ce as CourseEvent;
               return query.ToList();
           }
       }


       [WebGet(UriTemplate = "CourseEvents/Employee/{employeeID}")]
       [Description("CourseEvents for a specific employee")]
       public List<CourseEvent> CourseEventsByEmployeeID(string employeeID)
       {
           using (var context = new wsa500_103a06Entities())
           {
               int lookupID = int.Parse(employeeID);
               context.ContextOptions.ProxyCreationEnabled = false;
               var query = from ce in context.CourseEvents
                           where ce.CourseInstance.EmployeeID == lookupID
                           select ce;
               return query.ToList();
           }
       }

        
       [WebGet(UriTemplate = "JobTitle/Employee/{employeeID}")]
       [Description("Jobtitle for a specific Employee")]
        public List<JobTitle> jobTitlesByEmployeeID(string employeeID)
       {
           using (var context = new wsa500_103a06Entities())
           {
               int lookupID = int.Parse(employeeID);
               context.ContextOptions.ProxyCreationEnabled = false;
               var query = from e in context.Employees
                           where e.EmployeeID == lookupID
                           select e.JobTitle;
               return query.ToList();
           }
       }

      
       [WebGet(UriTemplate = "JobDuties/Employee/{employeeID}")]
       [Description("JobDuties for a specific Employee")]
           public List<JobDuty> jobDutiesByEmployeeID(string employeeID)
       {
           using (var context = new wsa500_103a06Entities())
           {
               int lookupID = int.Parse(employeeID);
               context.ContextOptions.ProxyCreationEnabled = false;
               var query = from e in context.Employees
                           from j in e.JobDuties
                           where e.EmployeeID == lookupID
                           select j;
               return query.ToList();
           }
       }

       [WebGet(UriTemplate = "Semester/SemesterNumber/{semesterNumber}")]
       [Description("Semester by semester number")]
            public Semester semesterBySemesterNumber(string semesterNumber)
       {
           using (var context = new wsa500_103a06Entities())
           {
               int lookupID = int.Parse(semesterNumber);
               context.ContextOptions.ProxyCreationEnabled = false;
               var query = from s in context.Semesters
                           where s.SemesterNumber == lookupID
                           select s;
               return query.ToList().First();
           }
       }

    }
}
